import asyncio
import logging
import sys
import threading
from concurrent.futures import Future
from functools import wraps
from typing import Optional, Self, Coroutine, Any, TypeVar

logger = logging.getLogger(__name__)
def singleton(cls):
    """单例装饰器"""
    instances = {}
    @wraps(cls)
    def get_instance(*args, **kwargs):
        if cls not in instances:
            obj = cls(*args, **kwargs)
            start = getattr(obj, "start", None)
            if callable(start):
                start()
            instances[cls] = obj
        return instances[cls]

    return get_instance

T = TypeVar("T")

@singleton
class RegistryThreadLoop:
    """服务注册线程循环"""
    def __init__(self):
        self._thread: Optional[threading.Thread] = None
        self._loop: Optional[asyncio.AbstractEventLoop] = None
        self._ready_event = threading.Event()  # 用于信令，确保线程已启动

    @staticmethod
    def create_standard_asyncio_loop():
        if sys.platform.startswith("linux") or sys.platform.startswith("darwin"):
            # Linux / macOS 明确使用 SelectorEventLoop（可完全绕开 uvloop）
            return asyncio.SelectorEventLoop()
        else:
            # Windows（使用 ProactorEventLoop 是更标准的）
            return asyncio.ProactorEventLoop()

    def _run_loop(self):
        try:
            self._loop = self.create_standard_asyncio_loop()
            asyncio.set_event_loop(self._loop)

            # 发送 "准备就绪" 信号
            self._ready_event.set()
            # 永久运行循环
            self._loop.run_forever()
        except Exception as e:
            logger.error(f"Registry worker thread failed: {e}")
            self._ready_event.set()  # 即使失败也要取消阻塞
        finally:
            if self._loop.is_running():
                self._loop.close()
            logger.info("Registry worker loop exited.")

    def start(self):
        """启动工作线程。"""
        if self._thread:
            logger.warning("Registry worker thread already started.")
            return

        logger.info("Starting Registry worker thread...")
        self._thread = threading.Thread(target=self._run_loop, daemon=True, name="RegistryWorker")
        self._thread.start()

        # 阻塞，直到 _run_loop 发出 "准备就绪" 信号
        self._ready_event.wait(timeout=10)

    async def run(self, coro: Coroutine[Any, Any, T]) -> T:
        """
        在工作线程中安全地运行一个协程，并异步等待其结果。

        Args:
            coro: 要在工作线程中执行的协程。

        Returns:
            协程的返回值 (类型为 T)。
        """
        if not self._thread or not self._loop:
            raise RuntimeError("Registry worker thread not started.")

        try:
            caller_loop = asyncio.get_running_loop()
        except RuntimeError:
            raise RuntimeError("run() must be called from within a running asyncio loop.")

        queue = asyncio.Queue(1)

        def _thread_safe_callback(future: asyncio.Future):
            caller_loop.call_soon_threadsafe(queue.put_nowait, future)

        future_ = asyncio.run_coroutine_threadsafe(coro, self._loop)
        future_.add_done_callback(_thread_safe_callback)

        f_result: Future = await queue.get()
        queue.task_done()

        # .result() 将返回类型 T 或引发异常
        return f_result.result()



import logging
import os
from pathlib import Path
from typing import Optional, Dict, Any

from yamlpyconfig.utils import ExpressionHelper
from yamlpyconfig.local_manage import LocalConfigLoader
from yamlpyconfig.utils import ConfigMerge

logger = logging.getLogger(__name__)

class LocalConfigManager:
    """
    Local configuration manager.
    默认获取 config_dir / application.yaml 以及 config_dir / application-{profile}.yaml 的本地文件配置。
    其中 profile 的获取逻辑是：
    1. 首先获取APP_PROFILE作为环境变量的值，如果存在返回
    2. 然后获取SPRING_PROFILES_ACTIVE座位环境变量的值，如果存在返回
    3. 最后获取application.yaml配置文件中的profile的配置，如果存在则返回
    如果以上三处都未获取到有效的profile，则不再尝试获取application-{profile}.yaml的配置。
    """

    def __init__(self, config_dir: Optional[str] = None):
        """
        Initialize local config manager.

        Args:
            config_dir: Directory containing config files. Defaults to current working directory.
        """
        self.config_dir = Path(config_dir) if config_dir else Path.cwd()

    def load_config(self) -> Dict[str, Any]:
            """Load local configuration.

            Args:
                profile: Profile to load. Defaults to None.

            Returns:
                A dictionary with the loaded configuration.
            """
            config: dict[str, Any] = LocalConfigLoader(self.config_dir).load_local_yaml("application")
            ExpressionHelper.resolve_config(config)

            profile = self._detect_profile(config)
            if profile:
                logger.info(f"Using profile: {profile}")
                config_profile = LocalConfigLoader(self.config_dir).load_local_yaml(f"application-{profile}")
                ConfigMerge.deep_merge(config, config_profile, False)
            ExpressionHelper.resolve_config(config)
            return config

    @staticmethod
    def _detect_profile(config) -> Optional[str]:
        """Detect profile from environment or config files."""
        # Check environment variable
        env_profile = os.getenv('APP_PROFILE', os.getenv('SPRING_PROFILES_ACTIVE'))
        if env_profile:
            return env_profile
        # Try to extract from local loader
        if "profile" in config:
            return config["profile"]
        return None


from typing import Dict, Any


class ConfigMerge:

    @staticmethod
    def deep_merge(base: Dict[str, Any], override: Dict[str, Any], list_merge: bool = False):
        """
        Deep merge two dictionaries.

        Args:
            base: Base dictionary (will be modified)
            override: Override dictionary
        """
        for key, value in override.items():
            if key in base:
                if isinstance(base[key], dict) and isinstance(value, dict):
                    # Both are dicts, merge recursively
                    ConfigMerge.deep_merge(base[key], value)
                elif isinstance(base[key], list) and isinstance(value, list) and list_merge:
                    # Both are lists, merge them
                    base[key] += value
                else:
                    # Override with new value
                    base[key] = value
            else:
                # New key, add it
                base[key] = value


from typing import Optional, Any

from pydantic import BaseModel, Field


class NacosConfigCacheItem(BaseModel):
    """Single Nacos configuration cache item."""
    data_id: str = Field(..., description="nacos data ID")
    config: Optional[dict[str, Any]] = Field(None, description="nacos configuration")


import importlib.util
import logging
from typing import Optional

from .models import AlgorithmEnum
from yamlpyconfig.config_cache import ConfigCache
from yamlpyconfig.local_manage.local_config_manager import LocalConfigManager
from yamlpyconfig.models import NacosConfig
from yamlpyconfig.utils.crypto.crypto_algorithm_factory import CryptoAlgorithmFactory
from .utils.crypto import CryptoHelper

logger = logging.getLogger(__name__)
class ConfigManager:
    """
    ConfigManager is a singleton class that manages the configuration of the application.
    It provides methods to get and set configuration values.
    """
    def __init__(self, config_dir: Optional[str] = None,
                 crypto_algorithm: Optional[AlgorithmEnum] = None, key: Optional[str] = None):
        """

        :param config_dir: 本地配置文件的根目录，包含application.yaml和application-{profile}.yaml的目录
        :param crypto_algorithm: 如果配置文件中包含{encrypted}加密数据，可以配置加密算法，用来解密
        :param key: 如果是sm2则传递私钥，如果是sm4则传递对称密钥
        """

        self._base_config = LocalConfigManager(config_dir).load_config()
        self._crypto_helper = None
        if crypto_algorithm and key:
            crypto_algorithm = CryptoAlgorithmFactory.create(crypto_algorithm, key_b64=key)
            self._crypto_helper = CryptoHelper(crypto_algorithm)
        self._cache = ConfigCache(base_config=self._base_config, crypto_helper=self._crypto_helper)
        self._nacos_manager = None


    async def start(self):
        if not importlib.util.find_spec("v2.nacos"):
            return
        from yamlpyconfig.nacos_manage.nacos_config_manager import NacosConfigManager
        if self._nacos_manager is None:
            try:
                nacos_config: NacosConfig = NacosConfig.load_nacos_config(self._cache.get_config())
                if nacos_config:
                    self._nacos_manager = NacosConfigManager(nacos_config, cache=self._cache)
                    await self._nacos_manager.start()
            except Exception as e:
                logger.error(f"Failed to connect to Nacos: {e}", exc_info=True)

    async def __aenter__(self):
        await self.start()
        return self

    async def stop(self):
        if self._nacos_manager:
            await self._nacos_manager.stop()

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.stop()

    def get_config(self):
        return self._cache.get_config()


from __future__ import annotations

from abc import ABC, abstractmethod
from typing import Dict


class CryptoAlgorithm(ABC):
    """加解密算法策略接口"""

    @abstractmethod
    def encrypt(self, plaintext: str) -> str:
        """明文 -> 密文（不含 {encrypted} 前缀）"""
        ...

    @abstractmethod
    def decrypt(self, ciphertext: str) -> str:
        """密文 -> 明文（不含 {encrypted} 前缀）"""
        ...

    @classmethod
    @abstractmethod
    def generate_keys(cls) -> Dict[str, str]:
        """
        生成该算法所需密钥。
        对称算法：{"key": "..."}；
        非对称算法：{"public_key": "...", "private_key": "..."}。
        """
        ...



import base64
from gmssl import sm2, func

from ..crypto_algorithm import CryptoAlgorithm


class SM2Algorithm(CryptoAlgorithm):
    """
    SM2 非对称加密算法
    public_key_b64:  Base64(HEX public key)
    private_key_b64: Base64(HEX private key)
    """

    def __init__(self, key_b64: str | None, public_key_b64: str | None = None):
        self.private_key_hex = base64.b64decode(key_b64).decode() if key_b64 else None
        self.public_key_hex = base64.b64decode(public_key_b64).decode() if public_key_b64 else None

        # 如果私钥存在但公钥不存在 → 自动推导
        if self.private_key_hex and not self.public_key_hex:
            sm2_crypt = sm2.CryptSM2(private_key=self.private_key_hex, public_key="")

            self.public_key_hex = sm2_crypt._kg(int(self.private_key_hex, 16), sm2_crypt.ecc_table["g"])

        self.sm2_crypt = sm2.CryptSM2(
            private_key=self.private_key_hex,
            public_key=self.public_key_hex
        )

    # ---------------- 加密（必须公钥） ---------------- #
    def encrypt(self, plaintext: str) -> str:
        if not self.public_key_hex:
            raise ValueError("SM2 encryption requires public_key")

        # encrypt 返回 bytes
        cipher_bytes: bytes = self.sm2_crypt.encrypt(plaintext.encode("utf-8"))
        # 统一封装成 Base64 字符串返回
        return base64.b64encode(cipher_bytes).decode("ascii")

    # ---------------- 解密（必须私钥） ---------------- #
    def decrypt(self, ciphertext: str) -> str:
        if not self.private_key_hex:
            raise ValueError("SM2 decryption requires private_key")

        # Base64 → bytes
        cipher_bytes: bytes = base64.b64decode(ciphertext)
        plain_bytes: bytes = self.sm2_crypt.decrypt(cipher_bytes)
        return plain_bytes.decode("utf-8")

    # ---------------- 密钥生成 ---------------- #
    @classmethod
    def generate_keys(cls) -> dict:
        """
        返回 Base64(HEX) 的公私钥
        """
        private_key_hex = func.random_hex(64)  # 256bit
        sm2_crypt = sm2.CryptSM2(private_key=private_key_hex, public_key="")

        public_key_hex = sm2_crypt._kg(int(private_key_hex, 16), sm2_crypt.ecc_table["g"])

        return {
            "private_key": base64.b64encode(private_key_hex.encode()).decode(),
            "public_key": base64.b64encode(public_key_hex.encode()).decode(),
        }



# ========================
# 3. NacosConfigManager：连接/初始化/监听
# ========================
from __future__ import annotations

import asyncio
import logging
from asyncio import TaskGroup, Task
from typing import Optional, Any, Dict, Iterator, Tuple, Mapping

import yaml
from v2.nacos import ClientConfigBuilder, GRPCConfig, NacosConfigService, ConfigParam

from yamlpyconfig.config_cache import ConfigCache
from yamlpyconfig.models import NacosConfig
from yamlpyconfig.nacos_manage.registry_thread_loop import RegistryThreadLoop

logger = logging.getLogger(__name__)
class NacosConfigManager:
    """
    负责：
    1. 用 NacosConfig 初始化 NacosClient；
    2. 根据 imports 顺序拉取初始配置并填充 ConfigCache；
    3. 若 auto_refresh=True，则为每个 dataId 注册监听器，在配置变更时更新 Cache。
    """

    def __init__(
        self,
        nacos_config: NacosConfig,
        cache: Optional[ConfigCache],
    ) -> None:
        self._cfg = nacos_config
        self._group = nacos_config.group or "DEFAULT_GROUP"
        self._cache = cache
        self._client: Optional[NacosConfigService] = None

    async def start(self):
        if self._client is None:
            # 初始化 Nacos 客户端
            self._client = await RegistryThreadLoop().run(self.__create_client(self._cfg))
            # 初始化配置
            await RegistryThreadLoop().run(self.__init_import_configs())
            # 如果需要自动刷新，则注册监听
            if self._cfg.auto_refresh:
                await RegistryThreadLoop().run(self.__register_listeners())


    async def stop(self):
        if self._client:
            await RegistryThreadLoop().run(self._client.shutdown())

    async def __aenter__(self):
        await self.start()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.stop()

    @staticmethod
    async def __create_client(cfg: NacosConfig) -> NacosConfigService:
        """
        创建 NacosClient。
        nacos-sdk-python 2.x 的参数名可能略有出入，可按实际 SDK 文档调整。
        """
        kwargs: Dict[str, Any] = {}
        if cfg.namespace:
            kwargs["namespace"] = cfg.namespace
        if cfg.username and cfg.password:
            kwargs["username"] = cfg.username
            kwargs["password"] = cfg.password
        if cfg.access_key and cfg.secret_key:
            kwargs["ak"] = cfg.access_key
            kwargs["sk"] = cfg.secret_key
        if cfg.endpoint:
            kwargs["endpoint"] = cfg.endpoint

        client_config = (ClientConfigBuilder()
                         .server_address(cfg.server_addr)
                         .username(cfg.username)
                         .password(cfg.password)
                         .namespace_id(cfg.namespace)
                         .access_key(cfg.access_key)
                         .secret_key(cfg.secret_key)
                         .log_level("INFO")
                         .grpc_config(GRPCConfig(grpc_timeout=5000))
                         .build())
        while True:
            try:
                config_client = await NacosConfigService.create_config_service(client_config)
                return config_client
            except Exception as e:
                logger.error(f"Nacos client init failed: {e}", exc_info=True)
                await asyncio.sleep(1)

    # ---------- 配置解析 & 初始化 ----------

    @staticmethod
    def __parse_yaml_config(content: Optional[str]) -> Dict[str, Any]:
        """
        解析 YAML 文本为 dict。
        - 若 content 为空或解析失败，则返回空 dict。
        - 若顶层不是 dict，则也返回空 dict（避免后续 merged.update 出问题）。
        """
        if not content:
            return {}
        try:
            data = yaml.safe_load(content)
            if isinstance(data, dict):
                return data
            return {}
        except Exception:
            # 这里按需选择：可以记录日志，或者抛自定义异常
            return {}

    async def __init_import_configs(self) -> None:
        """
        初始化加载所有 imports 中的 dataId 配置到 Cache。
        """
        imports = self._cfg.imports or []
        tasks: dict[str, Task] = {}
        async with TaskGroup() as tg:
            for item in imports:
                data_id = item.data_id
                task = tg.create_task(self.__get_data_id_config(data_id))
                tasks[data_id] = task
        for item in imports:
            data_id = item.data_id
            raw = tasks[data_id].result()
            cfg_dict = self.__parse_yaml_config(raw)
            await asyncio.to_thread(self._cache.set_config, data_id, cfg_dict)

    async def __get_data_id_config(self, data_id: str):
        while True:
            try:
                return await self._client.get_config(ConfigParam(data_id=data_id, group=self._group))
            except Exception as e:
                logger.error(f"Failed to get config for dataId: {data_id}: {e!r}", exc_info=True)
                await asyncio.sleep(1)

    # ---------- 监听配置变更 ----------

    async def __register_listeners(self) -> None:
        """
        为每个 imports.dataId 注册监听器。
        注意：具体监听 API 可能是 add_config_watcher 或 add_listener，
        这里用伪代码形式，使用时按实际 SDK 调整。
        """
        imports = self._cfg.imports or []
        async with TaskGroup() as tg:
            for item in imports:
                data_id = item.data_id
                tg.create_task(self.__register_single_listener(data_id))

    async def __register_single_listener(self, data_id: str) -> None:
        group = self._group

        async def _on_change(tenant, data_id, group, content) -> None:
            """
            Nacos 配置变化时的回调。
            只做两件事：
            1. 解析 YAML；
            2. 更新 Cache。
            """
            cfg_dict = None
            if content:
                cfg_dict = self.__parse_yaml_config(content)
            await asyncio.to_thread(self._cache.set_config, data_id, cfg_dict)

        # 这里根据 nacos-sdk-python 的实际接口调用
        # 官方 SDK 大致有类似 API：
        #   client.add_config_watcher(data_id, group, callback)
        # 或者：
        #   client.add_listener(data_id, group, callback)
        #
        # 假设使用 add_config_watcher：
        try:
            await self._client.add_listener(data_id, group, _on_change)  # 伪代码
        except Exception as e:
            logger.error(f"Failed to register Nacos listener (data_id={data_id}, group={group}): {e!r}", exc_info=True)

    # ---------- 对外访问接口 ----------

    @property
    def cache(self) -> ConfigCache:
        """暴露底层 Cache 对象（只读访问配置，写操作也可以）。"""
        return self._cache

    def get_config(self) -> Mapping[str, Any]:
        """获取合并后的最终配置快照（只读）。"""
        return self._cache.get_config()

    def get_nacos_configs(self) -> Iterator[Tuple[str, Optional[Dict[str, Any]]]]:
        """按 imports 顺序返回 (dataId, config)。"""
        return self._cache.get_nacos_configs()


from typing import Any, Dict

from .crypto_algorithm import CryptoAlgorithm
from ...models import AlgorithmEnum


class CryptoAlgorithmFactory:
    """算法策略工厂 + 注册表"""

    _registry: Dict[str, type[CryptoAlgorithm]] = {}

    @classmethod
    def register(cls, name: str, algo_cls: type[CryptoAlgorithm]) -> None:
        cls._registry[name.lower()] = algo_cls

    @classmethod
    def create(cls, name: AlgorithmEnum, **kwargs: Any) -> CryptoAlgorithm:
        algo_cls = cls._registry.get(name.value.lower())
        if algo_cls is None:
            raise ValueError(f"Unsupported algorithm: {name}")
        return algo_cls(**kwargs)  # type: ignore[arg-type]

    @classmethod
    def get_algorithms_class(cls, name: AlgorithmEnum):
        algo_cls = cls._registry.get(name.value.lower())
        return algo_cls

    @classmethod
    def list_algorithms(cls) -> list[str]:
        return list(cls._registry.keys())



import os
import re
from typing import Optional, Any


class ExpressionHelper:
    """
    表达式处理类，用于解析字符串中的环境变量占位符。
    """
    @staticmethod
    def _resolve_env_placeholder(s: str) -> Optional[str]:
        """
        解析字符串，匹配 ${key[:default_value]} 格式的占位符。
        如果环境变量存在，则返回其值；
        如果不存在且有默认值，则返回默认值；
        如果两者都不存在，则返回 None。
        """
        # 匹配形如 ${key} 或 ${key:default_value}
        pattern = re.compile(r"^\$\{([^:}]+)(?::(.*))?\}$")
        match = pattern.match(s.strip())
        if not match:
            return s  # 非占位符格式，原样返回（也可以返回 None 看你需求）

        key, default = match.groups()
        return os.getenv(key, default)

    @staticmethod
    def resolve_config(config: dict[str, Any]):
        """
        解析字符串中的所有环境变量占位符。
        """
        for key, value in config.items():
            if isinstance(value, str):
                resolved_value = ExpressionHelper._resolve_env_placeholder(value)
                if resolved_value is not None:
                    config[key] = resolved_value
            elif isinstance(value, dict):
                ExpressionHelper.resolve_config(value)




from typing import Optional, Dict, Any, Self

from .crypto_algorithm import CryptoAlgorithm
from ...models import AlgorithmEnum
from .crypto_algorithm_factory import CryptoAlgorithmFactory


class CryptoHelper:
    """
    业务层使用的加解密工具：
    - 负责识别/封装 `{encrypted}` 前缀
    - 持有一个具体的 CryptoAlgorithm 策略
    """

    ENCRYPTED_PREFIX = "{encrypted}"

    def __init__(self, algorithm: CryptoAlgorithm):
        self._algorithm = algorithm

    @classmethod
    def from_algorithm_name(cls, name: AlgorithmEnum, **kwargs: Any) -> Self:
        """工厂式构造：通过算法名 + 参数创建 CryptoHelper"""
        algo = CryptoAlgorithmFactory.create(name, **kwargs)
        return cls(algo)

    @staticmethod
    def strip_prefix(value: str, prefix: str) -> Optional[str]:
        """如果以 prefix 开头，返回去掉前缀后的内容，否则返回 None"""
        if value.startswith(prefix):
            return value[len(prefix):]
        return None

    def encrypt_value(self, value: str, with_prefix: bool = True) -> str:
        """
        对传入的明文进行加密。
        - 默认加 `{encrypted}` 前缀；
        - 如果 value 已经是带前缀的密文，可以根据需要选择是否重复加密。
        """
        ciphertext = self._algorithm.encrypt(value)
        if with_prefix:
            return f"{self.ENCRYPTED_PREFIX}{ciphertext}"
        return ciphertext

    def decrypt_value(self, value: str) -> str:
        """
        对传入的值进行解密：
        - 如果以 `{encrypted}` 开头，则截取后面的内容进行解密；
        - 否则认为是明文，直接返回。
        """
        inner = self.strip_prefix(value, self.ENCRYPTED_PREFIX)
        if inner is None:
            # 不带前缀，当作明文返回
            return value
        return self._algorithm.decrypt(inner)

    @staticmethod
    def generate_keys(algorithm_name: AlgorithmEnum) -> Dict[str, str]:
        """统一的密钥生成入口"""
        algo_cls = CryptoAlgorithmFactory.get_algorithms_class(algorithm_name)
        if algo_cls is None:
            raise ValueError(f"Unsupported algorithm: {algorithm_name}")
        return algo_cls.generate_keys()

    def decrypt_config_value(self, config: dict[str, Any] | None):
        if config is None:
            return
        for k, v in config.items():
            if isinstance(v, str):
                config[k] = self.decrypt_value(v)
            elif isinstance(v, dict):
                self.decrypt_config_value(v)




from .crypto_helper import CryptoHelper
from .impl.SM2Algorithm import SM2Algorithm as __SM2Algorithm
from .impl.SM4Algorithm import SM4Algorithm as __SM4Algorithm
from .crypto_algorithm_factory import CryptoAlgorithmFactory as __CryptoAlgorithmFactory
from ...models import AlgorithmEnum

__CryptoAlgorithmFactory.register(AlgorithmEnum.SM4.value, __SM4Algorithm)
__CryptoAlgorithmFactory.register(AlgorithmEnum.SM2.value, __SM2Algorithm)






import base64
from gmssl.sm4 import CryptSM4, SM4_ENCRYPT, SM4_DECRYPT

from ..crypto_algorithm import CryptoAlgorithm


class SM4Algorithm(CryptoAlgorithm):
    """
    SM4 对称加密算法（默认 ECB 模式）
    key: 128-bit（16字节）Base64 字符串
    """
    def __init__(self, key_b64: str, mode: str = "ECB", iv_b64: str | None = None):
        self.key = base64.b64decode(key_b64)
        self.mode = mode.upper()
        self.iv = base64.b64decode(iv_b64) if iv_b64 else None

        if len(self.key) != 16:
            raise ValueError("SM4 key must be 16 bytes (128-bit).")

        if self.mode not in ("ECB", "CBC"):
            raise ValueError("SM4 mode must be ECB or CBC")

        if self.mode == "CBC" and self.iv is None:
            raise ValueError("CBC mode requires iv")

    # ---------------- 加密 ---------------- #
    def encrypt(self, plaintext: str) -> str:
        crypt = CryptSM4()
        crypt.set_key(self.key, SM4_ENCRYPT)

        if self.mode == "ECB":
            cipher_bytes = crypt.crypt_ecb(plaintext.encode())
        else:
            cipher_bytes = crypt.crypt_cbc(self.iv, plaintext.encode())

        return base64.b64encode(cipher_bytes).decode()

    # ---------------- 解密 ---------------- #
    def decrypt(self, ciphertext: str) -> str:
        crypt = CryptSM4()
        crypt.set_key(self.key, SM4_DECRYPT)

        cipher_bytes = base64.b64decode(ciphertext)

        if self.mode == "ECB":
            plain_bytes = crypt.crypt_ecb(cipher_bytes)
        else:
            plain_bytes = crypt.crypt_cbc(self.iv, cipher_bytes)

        return plain_bytes.decode()

    # ---------------- 密钥生成 ---------------- #
    @classmethod
    def generate_keys(cls) -> dict:
        """
        生成 128bit SM4 密钥（Base64 字符串）
        若需要 CBC 模式，可额外生成16字节IV
        """
        import os
        key = os.urandom(16)
        iv = os.urandom(16)

        return {
            "key": base64.b64encode(key).decode(),
            "iv": base64.b64encode(iv).decode(),
        }



from .nacos_config import NacosConfig, ImportConfigItem
from .nacos_config_cache_item import NacosConfigCacheItem
from .algorithm_enum import AlgorithmEnum


__all__ = [
    "AlgorithmEnum",
    "NacosConfig"
]


from __future__ import annotations

from pathlib import Path
from typing import Any, Dict, Iterable, Optional

import yaml


class LocalConfigLoader:
    """Local YAML configuration file loader."""

    def __init__(self, config_dir: Optional[str|Path] = None):
        """
        Initialize local_manage config loader.

        Args:
            config_dir: Directory containing config files. Defaults to current working directory.
        """
        if isinstance(config_dir, str):
            self.config_dir = Path(config_dir) if config_dir else Path.cwd()
        else:
            self.config_dir = config_dir

    @staticmethod
    def _candidate_paths(base: Path, name: str) -> Iterable[Path]:
        """Yield possible file paths for the given base directory and name.

        If the provided name has no YAML extension, try common variants.
        """
        p = base / name
        if p.suffix.lower() in {".yml", ".yaml"}:
            yield p
            return
        # Try with typical YAML suffixes
        yield p.with_suffix(".yaml")
        yield p.with_suffix(".yml")


    def load_local_yaml(self, filename: str) -> Dict[str, Any]:
        """Load a local_manage YAML configuration file.

        Parameters
        - config_dir: The directory containing the configuration file.
        - filename: The configuration file name, with or without .yaml/.yml.

        Returns
        - A dictionary with the parsed YAML content (empty dict if file is empty).

        Raises
        - FileNotFoundError: If no matching file is found.
        - yaml.YAMLError: If the YAML content is invalid.
        """
        if not self.config_dir.exists() or not self.config_dir.is_dir():
            raise FileNotFoundError(f"Config directory does not exist: {self.config_dir}")

        for path in self._candidate_paths(self.config_dir, filename):
            if path.exists() and path.is_file():
                with path.open("r", encoding="utf-8") as f:
                    data = yaml.safe_load(f)
                return data or {}

        tried = ", ".join(str(p) for p in self._candidate_paths(self.config_dir, filename))
        raise FileNotFoundError(
            f"Config file not found. Tried: {tried}"
        )




import copy
from threading import RLock
from types import MappingProxyType
from typing import Dict, Any, Optional, Mapping, List, Iterator, Tuple

from yamlpyconfig.models import NacosConfigCacheItem
from yamlpyconfig.utils import ConfigMerge
from yamlpyconfig.utils.crypto import CryptoHelper


class ConfigCache:
    """
    负责缓存本地配置 + Nacos 多个 dataId 的配置，并合并成最终配置。
    - 写路径：加锁 + 重建 merged_config（copy-on-write）
    - 读路径：读只读快照，不加锁（或极短锁）
    """

    def __init__(self, base_config: Optional[Dict[str, Any]] = None, crypto_helper: CryptoHelper = None) -> None:
        self._lock = RLock()
        # 对外暴露的最终配置快照（不可变）
        self._merged_config: Mapping[str, Any] = MappingProxyType({})
        self._base_config: dict[str, Any] = {}
        self._crypto_helper: CryptoHelper|None = crypto_helper
        self._order: List[str] = []  # imports 顺序
        self._items: Dict[str, NacosConfigCacheItem] = {}
        self.set_base_config(base_config)


    def set_base_config(self, base_config: Dict[str, Any]) -> None:
        """更新本地基础配置，并重建 merged_config。"""
        with self._lock:
            self._base_config = dict(base_config or {})
            if self._crypto_helper:
                self._crypto_helper.decrypt_config_value(self._base_config)
            self._rebuild_merged_unlocked()

    def set_config(self, data_id: str, config: Optional[Dict[str, Any]]) -> None:
        """
        设置/更新某个 dataId 的配置。
        - 若 data_id 不存在，则追加到顺序末尾；
        - 若已存在，仅更新对应的 config。
        """
        with self._lock:
            if self._crypto_helper:
                self._crypto_helper.decrypt_config_value(config)
            if data_id not in self._items:
                self._order.append(data_id)
                self._items[data_id] = NacosConfigCacheItem(data_id=data_id, config=config)
            else:
                # pydantic v2: 可以直接更新属性，也可以 model_copy(update=...)
                item = self._items[data_id]
                item.config = config

            self._rebuild_merged_unlocked()

    def _rebuild_merged_unlocked(self) -> None:
        """
        在锁内重建 merged_config。
        合并策略（浅合并）：
        1. 先拷贝 base_config；
        2. 按 _order 顺序应用每个 dataId 的 config，后者覆盖前者同名 key。
        """
        merged: Dict[str, Any] = copy.deepcopy(self._base_config)

        for data_id in self._order:
            item = self._items.get(data_id)
            if not item or not item.config:
                continue
            cfg = item.config
            if not isinstance(cfg, dict):
                # 若配置不是 dict（比如 yaml 顶层不是映射），可以选择跳过或抛错
                continue
            ConfigMerge.deep_merge(merged, cfg)

        # copy-on-write: 用新的 dict 替换旧引用，并加只读包装
        self._merged_config = MappingProxyType(merged)

    def get_nacos_configs(self) -> Iterator[Tuple[str, Optional[Dict[str, Any]]]]:
        """
        按 imports 顺序依次返回 (dataId, config) 的只读快照。
        为了避免在长时间迭代时持有锁，这里使用“快照”：
        - 在锁内复制 _order 和 _items 的当前状态；
        - 解锁后再遍历快照。
        """
        with self._lock:
            order_snapshot = tuple(self._order)
            items_snapshot = {k: v.config for k, v in self._items.items()}

        for data_id in order_snapshot:
            yield data_id, items_snapshot.get(data_id)

    def get_config(self) -> Mapping[str, Any]:
        """
        返回最终合并后的配置快照（只读 Mapping）。
        注意：返回的是只读的 mappingproxy，对外修改会抛异常。
        """

        return self._merged_config



from typing import Optional, Any

from pydantic import BaseModel, Field, field_validator
from typing import Self

class ImportConfigItem(BaseModel):
    """Single import configuration item."""
    data_id: str = Field(..., alias="data-id", description="Import data_id, e.g., 'gateway.yaml'")

    @field_validator("data_id")
    def validate_source(cls, v):
        if not v or ('yaml' not in v and 'yml' not in v):
            raise ValueError("data_id must be in format 'name.yaml', e.g., 'gateway.yaml'")
        return v

class NacosConfig(BaseModel):
    """Nacos configuration model."""
    server_addr: str = Field(..., alias="server-addr", description="Nacos server address")
    namespace: Optional[str] = Field(None, alias="namespace", description="Nacos namespace")
    group: Optional[str] = Field("DEFAULT_GROUP", alias="group", description="Nacos group")
    username: Optional[str] = Field(None, alias="username", description="Nacos username")
    password: Optional[str] = Field(None, alias="password", description="Nacos password")
    access_key: Optional[str] = Field(None, alias="access-key", description="Nacos access key")
    secret_key: Optional[str] = Field(None, alias="secret-key", description="Nacos secret key")
    endpoint: Optional[str] = Field(None, alias="endpoint", description="Nacos endpoint for ACM")
    auto_refresh: bool = Field(True, alias="auto-refresh", description="Whether to auto refresh config")
    imports: Optional[list[ImportConfigItem]] = Field(None, alias="imports", description="List of import configurations")

    @classmethod
    def load_nacos_config(cls, config: dict[str, Any]) -> Optional[Self]:
        """Load Nacos configuration from a dict."""
        if "config-sources" in config and "nacos" in config["config-sources"]:
            return NacosConfig.model_validate(config["config-sources"]["nacos"])
        return None


